package com.netiq.nam.oauth.helpers;

import com.google.api.client.http.HttpResponse;
import org.glassfish.jersey.server.ExtendedUriInfo;

import javax.annotation.Priority;
import javax.inject.Inject;
import javax.servlet.http.HttpServletRequest;
import javax.ws.rs.Priorities;
import javax.ws.rs.container.ContainerRequestContext;
import javax.ws.rs.container.ContainerRequestFilter;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.Response;
import javax.ws.rs.ext.Provider;
import java.io.IOException;
import java.util.Optional;

/**
 * Created by tsureshkumar on 12/7/14.
 */
@Provider
@Priority(Priorities.AUTHENTICATION)
@OAuth2Protected
public class TokenFilter implements ContainerRequestFilter {
    public static final String AUTHORIZATION_HEADER = "Authorization";

    @Inject
    HttpServletRequest request;

    @Override
    public void filter(ContainerRequestContext ctx) throws IOException {
        final ExtendedUriInfo extendendUriInfo = (ExtendedUriInfo) ctx.getUriInfo();
        OAuth2Protected annot = extendendUriInfo
                .getMatchedResourceMethod()
                .getInvocable()
                .getHandlingMethod().getAnnotation(OAuth2Protected.class);
        if (annot != null) {
            // only protect REST methods with @OAuth2Protected annotated
            Optional<Token> token = checkForOAuthHeader(ctx, annot);
            if (!token.isPresent() ||
                    token.get().isExpired()) {
                ctx.abortWith(Response.status(401)
                        .entity("{\"error\" : \"oauth authentication required\"}")
                        .type(MediaType.APPLICATION_JSON_TYPE)
                        .header("WWW-Authenticate", "error=\"OAuth bearer token required.\"")
                        .build());
            } else {
                ctx.setSecurityContext(new OAuth2SecurityContext(token.get(), request.isSecure()));
            }
        }
    }

    private Optional<Token> checkForOAuthHeader(ContainerRequestContext ctx, OAuth2Protected annot) {
        String authzHeader = ctx.getHeaderString(AUTHORIZATION_HEADER);
        Optional<String> tokenString = getoAuth2Token(authzHeader);
        Optional<Token> token = validateToken(tokenString);
        return token;
    }

    /*
    * this is a simple method which makes call everytime to Authorization Server's endpoint to validate
    * token.  This should be further improved to cache the resturned token until it expires.
     */
    private Optional<Token> validateToken(Optional<String> tokenOpt) {
        return tokenOpt.flatMap(tokenStr -> {
            try {
                HttpResponse response = new Http()
                        .insecureGet(OAuth2Properties.tokenInfoEndpoint(), tokenStr);
                if (response.getStatusCode() != 200)
                    return Optional.empty();
                else {
                    Token token = response
                            .parseAs(Token.class);
                    token.setReceivedTime(System.currentTimeMillis());
                    return Optional.of(token);
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
            return Optional.empty();
        });
    }

    private Optional<String> getoAuth2Token(String authzHeader) {
        if (authzHeader == null)
            return Optional.empty();
        String[] split = authzHeader.split(" ", 2);
        if (split.length < 2 ||
                !split[0].equalsIgnoreCase("Bearer"))
            return Optional.empty();
        String token1 = split[1];
        return Optional.of(token1);
    }

}
